@@ -1,59 +1,48 @@
 import os
 import datetime
 
+import redis
 from celery import Celery, group
-from traitlets import Type, Unicode, Integer, List, default, Callable, Bool
+from traitlets import (
+    Type,
+    Unicode,
+    Integer,
+    List,
+    default,
+    Callable,
+    Bool,
+    validate,
+    TraitError,
+)
 from traitlets.config import LoggingConfigurable
 from sqlalchemy.pool import NullPool
-from conda.models.match_spec import MatchSpec
 
-from conda_store_server import orm, utils, storage, schema, api, conda
+from conda_store_server import orm, utils, storage, schema, api, conda, environment
 
 
 def conda_store_validate_specification(
     conda_store: "CondaStore", namespace: str, specification: schema.CondaSpecification
 ) -> schema.CondaSpecification:
-
-    # ============== MODIFICATION =================
-    # CondaStore.conda_default_channels
-    if len(specification.channels) == 0:
-        specification.channels = conda_store.conda_default_channels.copy()
-
-    # CondaStore.conda_default_packages
-    if len(specification.dependencies) == 0:
-        specification.dependencies = conda_store.conda_default_packages.copy()
-
-    # CondaStore.conda_included_packages
-    dependency_names = set(
-        MatchSpec(_).name for _ in specification.dependencies if isinstance(_, str)
-    )
-    for package in set(conda_store.conda_included_packages) - dependency_names:
-        specification.dependencies.append(package)
-
-    # ================ VALIDATION ===============
-    normalized_conda_channels = set(
-        conda.normalize_channel_name(conda_store.conda_channel_alias, _)
-        for _ in specification.channels
+    specification = environment.validate_environment_channels(
+        specification,
+        conda_store.conda_channel_alias,
+        conda_store.conda_default_channels,
+        conda_store.conda_allowed_channels,
     )
 
-    # validate that all listed channels in specification are allowed
-    normalized_conda_allowed_channels = set(
-        conda.normalize_channel_name(conda_store.conda_channel_alias, _)
-        for _ in conda_store.conda_allowed_channels
+    specification = environment.validate_environment_pypi_packages(
+        specification,
+        conda_store.pypi_default_packages,
+        conda_store.pypi_included_packages,
+        conda_store.pypi_required_packages,
     )
-    if not (normalized_conda_channels <= normalized_conda_allowed_channels):
-        raise ValueError(
-            f"Conda channels {normalized_conda_channels - normalized_conda_allowed_channels} not allowed in specification"
-        )
 
-    # validate that required conda package are in specification
-    dependency_names = set(
-        MatchSpec(_).name for _ in specification.dependencies if isinstance(_, str)
+    specification = environment.validate_environment_conda_packages(
+        specification,
+        conda_store.conda_default_packages,
+        conda_store.conda_included_packages,
+        conda_store.conda_required_packages,
     )
-    if not (set(conda_store.conda_required_packages) <= dependency_names):
-        raise ValueError(
-            f"Conda packages {conda_store.conda_required_packages - dependency_names} required and missing from specification"
-        )
 
     return specification
 
@@ -135,12 +124,55 @@ class CondaStore(LoggingConfigurable):
         config=True,
     )
 
+    pypi_default_packages = List(
+        [],
+        help="PyPi packages that included by default if none are included",
+        config=True,
+    )
+
+    pypi_required_packages = List(
+        [],
+        help="PyPi packages that are required to be within environment specification. Will raise a validation error is package not in specification",
+        config=True,
+    )
+
+    pypi_included_packages = List(
+        [],
+        help="PyPi packages that auto included within environment specification. Will not raise a validation error if package not in specification and will be auto added",
+        config=True,
+    )
+
+    conda_max_solve_time = Integer(
+        5 * 60,  # 5 minute
+        help="Maximum time in seconds to allow for solving a given conda environment",
+        config=True,
+    )
+
     database_url = Unicode(
         "sqlite:///conda-store.sqlite",
         help="url for the database. e.g. 'sqlite:///conda-store.sqlite' tables will be automatically created if they do not exist",
         config=True,
     )
 
+    redis_url = Unicode(
+        help="Redis connection url in form 'redis://:<password>@<hostname>:<port>/0'. Connection is used by Celery along with Conda-Store internally",
+        config=True,
+    )
+
+    @default("redis_url")
+    def _default_redis(self):
+        raise TraitError("c.CondaStore.redis_url Redis connection url is required")
+
+    @validate("redis_url")
+    def _check_redis(self, proposal):
+        try:
+            self.redis.ping()
+        except Exception:
+            raise TraitError(
+                f'c.CondaStore.redis_url unable to connect with Redis database at "{self.redis_url}"'
+            )
+        return proposal.value
+
     celery_broker_url = Unicode(
         help="broker url to use for celery tasks",
         config=True,
@@ -148,10 +180,10 @@ class CondaStore(LoggingConfigurable):
 
     build_artifacts = List(
         [
-            orm.BuildArtifactType.LOCKFILE,
-            orm.BuildArtifactType.YAML,
-            orm.BuildArtifactType.CONDA_PACK,
-            orm.BuildArtifactType.DOCKER_MANIFEST,
+            schema.BuildArtifactType.LOCKFILE,
+            schema.BuildArtifactType.YAML,
+            schema.BuildArtifactType.CONDA_PACK,
+            schema.BuildArtifactType.DOCKER_MANIFEST,
         ],
         help="artifacts to build in conda-store. By default all of the artifacts",
         config=True,
@@ -159,9 +191,9 @@ class CondaStore(LoggingConfigurable):
 
     build_artifacts_kept_on_deletion = List(
         [
-            orm.BuildArtifactType.LOGS,
-            orm.BuildArtifactType.LOCKFILE,
-            orm.BuildArtifactType.YAML,
+            schema.BuildArtifactType.LOGS,
+            schema.BuildArtifactType.LOCKFILE,
+            schema.BuildArtifactType.YAML,
         ],
         help="artifacts to keep on build deletion",
         config=True,
@@ -175,7 +207,7 @@ class CondaStore(LoggingConfigurable):
 
     @default("celery_broker_url")
     def _default_celery_broker_url(self):
-        return f"sqla+{self.database_url}"
+        return self.redis_url
 
     celery_results_backend = Unicode(
         help="backend to use for celery task results",
@@ -184,12 +216,18 @@ class CondaStore(LoggingConfigurable):
 
     @default("celery_results_backend")
     def _default_celery_results_backend(self):
-        return f"db+{self.database_url}"
+        return self.redis_url
 
     default_namespace = Unicode(
         "default", help="default namespace for conda-store", config=True
     )
 
+    filesystem_namespace = Unicode(
+        "filesystem",
+        help="namespace to use for environments picked up via `CondaStoreWorker.watch_paths` on the filesystem",
+        config=True,
+    )
+
     default_uid = Integer(
         os.getuid(),
         help="default uid to assign to built environments",
@@ -240,6 +278,13 @@ class CondaStore(LoggingConfigurable):
         # https://docs.sqlalchemy.org/en/14/orm/contextual.html
         return self.session_factory()
 
+    @property
+    def redis(self):
+        if hasattr(self, "_redis"):
+            return self._redis
+        self._redis = redis.Redis.from_url(self.redis_url)
+        return self._redis
+
     @property
     def configuration(self):
         return orm.CondaStoreConfiguration.configuration(self.db)
@@ -319,6 +364,39 @@ class CondaStore(LoggingConfigurable):
                 self.db.add(conda_channel)
                 self.db.commit()
 
+    def register_solve(self, specification: schema.CondaSpecification):
+        specification_model = self.validate_specification(
+            conda_store=self,
+            namespace="solve",
+            specification=specification,
+        )
+        specification_sha256 = utils.datastructure_hash(specification_model.dict())
+        specification = api.get_specification(self.db, sha256=specification_sha256)
+        if specification is None:
+            self.log.info(
+                f"specification name={specification_model.name} sha256={specification_sha256} registered"
+            )
+            specification = orm.Specification(specification_model.dict())
+            self.db.add(specification)
+            self.db.commit()
+        else:
+            self.log.debug(
+                f"specification name={specification_model.name} sha256={specification_sha256} already registered"
+            )
+
+        solve_model = orm.Solve(specification_id=specification.id)
+        self.db.add(solve_model)
+        self.db.commit()
+
+        # must import tasks after a celery app has been initialized
+        from conda_store_server.worker import tasks
+
+        task = tasks.task_solve_conda_environment.apply_async(
+            args=[solve_model.id], time_limit=self.conda_max_solve_time
+        )
+
+        return task, solve_model.id
+
     def register_environment(
         self, specification: dict, namespace: str = None, force_build=False
     ):
@@ -396,11 +474,11 @@ class CondaStore(LoggingConfigurable):
         from conda_store_server.worker import tasks
 
         artifact_tasks = []
-        if orm.BuildArtifactType.YAML in self.build_artifacts:
+        if schema.BuildArtifactType.YAML in self.build_artifacts:
             artifact_tasks.append(tasks.task_build_conda_env_export.si(build.id))
-        if orm.BuildArtifactType.CONDA_PACK in self.build_artifacts:
+        if schema.BuildArtifactType.CONDA_PACK in self.build_artifacts:
             artifact_tasks.append(tasks.task_build_conda_pack.si(build.id))
-        if orm.BuildArtifactType.DOCKER_MANIFEST in self.build_artifacts:
+        if schema.BuildArtifactType.DOCKER_MANIFEST in self.build_artifacts:
             artifact_tasks.append(tasks.task_build_conda_docker.si(build.id))
 
         (
@@ -423,7 +501,7 @@ class CondaStore(LoggingConfigurable):
                 f"environment namespace={namespace} name={name} does not exist"
             )
 
-        if build.status != orm.BuildStatus.COMPLETED:
+        if build.status != schema.BuildStatus.COMPLETED:
             raise utils.CondaStoreError(
                 "cannot update environment to build id since not completed"
             )
@@ -449,9 +527,9 @@ class CondaStore(LoggingConfigurable):
 
         utcnow = datetime.datetime.utcnow()
         namespace.deleted_on = utcnow
-        for environment in namespace.environments:
-            environment.deleted_on = utcnow
-            for build in environment.builds:
+        for environment_orm in namespace.environments:
+            environment_orm.deleted_on = utcnow
+            for build in environment_orm.builds:
                 build.deleted_on = utcnow
         self.db.commit()
 
@@ -484,7 +562,10 @@ class CondaStore(LoggingConfigurable):
 
     def delete_build(self, build_id):
         build = api.get_build(self.db, build_id)
-        if build.status not in [orm.BuildStatus.FAILED, orm.BuildStatus.COMPLETED]:
+        if build.status not in [
+            schema.BuildStatus.FAILED,
+            schema.BuildStatus.COMPLETED,
+        ]:
             raise utils.CondaStoreError(
                 "cannot delete build since not finished building"
             )
