@@ -1,5 +1,4 @@
 import os
-import enum
 import datetime
 import shutil
 import itertools
@@ -23,7 +22,7 @@ from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy import create_engine
 
-from conda_store_server import utils
+from conda_store_server import utils, schema
 from conda_store_server.environment import validate_environment
 from conda_store_server.conda import download_repodata
 
@@ -31,23 +30,6 @@ from conda_store_server.conda import download_repodata
 Base = declarative_base()
 
 
-class BuildArtifactType(enum.Enum):
-    DIRECTORY = "DIRECTORY"
-    LOCKFILE = "LOCKFILE"
-    LOGS = "LOGS"
-    YAML = "YAML"
-    CONDA_PACK = "CONDA_PACK"
-    DOCKER_BLOB = "DOCKER_BLOB"
-    DOCKER_MANIFEST = "DOCKER_MANIFEST"
-
-
-class BuildStatus(enum.Enum):
-    QUEUED = "QUEUED"
-    BUILDING = "BUILDING"
-    COMPLETED = "COMPLETED"
-    FAILED = "FAILED"
-
-
 class Namespace(Base):
     """Namespace for resources"""
 
@@ -82,6 +64,35 @@ class Specification(Base):
     created_on = Column(DateTime, default=datetime.datetime.utcnow)
 
     builds = relationship("Build", back_populates="specification")
+    solves = relationship("Solve", back_populates="specification")
+
+
+solve_conda_package = Table(
+    "solve_conda_package",
+    Base.metadata,
+    Column("solve_id", ForeignKey("solve.id", ondelete="CASCADE"), primary_key=True),
+    Column(
+        "conda_package_id",
+        ForeignKey("conda_package.id", ondelete="CASCADE"),
+        primary_key=True,
+    ),
+)
+
+
+class Solve(Base):
+    """A solve for a particular specification"""
+
+    __tablename__ = "solve"
+
+    id = Column(Integer, primary_key=True)
+    specification_id = Column(Integer, ForeignKey("specification.id"), nullable=False)
+    specification = relationship(Specification, back_populates="solves")
+
+    scheduled_on = Column(DateTime, default=datetime.datetime.utcnow)
+    started_on = Column(DateTime, default=None)
+    ended_on = Column(DateTime, default=None)
+
+    packages = relationship("CondaPackage", secondary=solve_conda_package)
 
 
 build_conda_package = Table(
@@ -114,7 +125,7 @@ class Build(Base):
 
     packages = relationship("CondaPackage", secondary=build_conda_package)
 
-    status = Column(Enum(BuildStatus), default=BuildStatus.QUEUED)
+    status = Column(Enum(schema.BuildStatus), default=schema.BuildStatus.QUEUED)
     size = Column(BigInteger, default=0)
     scheduled_on = Column(DateTime, default=datetime.datetime.utcnow)
     started_on = Column(DateTime, default=None)
@@ -198,28 +209,28 @@ class Build(Base):
     @hybrid_property
     def has_lockfile(self):
         return any(
-            artifact.artifact_type == BuildArtifactType.LOCKFILE
+            artifact.artifact_type == schema.BuildArtifactType.LOCKFILE
             for artifact in self.build_artifacts
         )
 
     @hybrid_property
     def has_yaml(self):
         return any(
-            artifact.artifact_type == BuildArtifactType.YAML
+            artifact.artifact_type == schema.BuildArtifactType.YAML
             for artifact in self.build_artifacts
         )
 
     @hybrid_property
     def has_conda_pack(self):
         return any(
-            artifact.artifact_type == BuildArtifactType.CONDA_PACK
+            artifact.artifact_type == schema.BuildArtifactType.CONDA_PACK
             for artifact in self.build_artifacts
         )
 
     @hybrid_property
     def has_docker_manifest(self):
         return any(
-            artifact.artifact_type == BuildArtifactType.DOCKER_MANIFEST
+            artifact.artifact_type == schema.BuildArtifactType.DOCKER_MANIFEST
             for artifact in self.build_artifacts
         )
 
@@ -234,7 +245,7 @@ class BuildArtifact(Base):
     build_id = Column(Integer, ForeignKey("build.id"))
     build = relationship(Build, back_populates="build_artifacts")
 
-    artifact_type = Column(Enum(BuildArtifactType), nullable=False)
+    artifact_type = Column(Enum(schema.BuildArtifactType), nullable=False)
 
     key = Column(Unicode(255))
 
