@@ -1,10 +1,10 @@
+import re
 import datetime
 import enum
-from typing import List, Optional, Union, Dict
+from typing import List, Optional, Union, Dict, Any
 import functools
 from pkg_resources import Requirement
 
-from conda.models.match_spec import MatchSpec
 from pydantic import BaseModel, Field, constr, validator
 
 
@@ -18,6 +18,17 @@ def _datetime_factory(offset: datetime.timedelta):
 #########################
 
 
+class Permissions(enum.Enum):
+    ENVIRONMENT_CREATE = "environment:create"
+    ENVIRONMENT_READ = "environment::read"
+    ENVIRONMENT_UPDATE = "environment::update"
+    ENVIRONMENT_DELETE = "environment::delete"
+    BUILD_DELETE = "build::delete"
+    NAMESPACE_CREATE = "namespace::create"
+    NAMESPACE_READ = "namespace::read"
+    NAMESPACE_DELETE = "namespace::delete"
+
+
 class AuthenticationToken(BaseModel):
     exp: datetime.datetime = Field(
         default_factory=functools.partial(_datetime_factory, datetime.timedelta(days=1))
@@ -83,12 +94,29 @@ class Specification(BaseModel):
         orm_mode = True
 
 
+class BuildArtifactType(enum.Enum):
+    DIRECTORY = "DIRECTORY"
+    LOCKFILE = "LOCKFILE"
+    LOGS = "LOGS"
+    YAML = "YAML"
+    CONDA_PACK = "CONDA_PACK"
+    DOCKER_BLOB = "DOCKER_BLOB"
+    DOCKER_MANIFEST = "DOCKER_MANIFEST"
+
+
+class BuildStatus(enum.Enum):
+    QUEUED = "QUEUED"
+    BUILDING = "BUILDING"
+    COMPLETED = "COMPLETED"
+    FAILED = "FAILED"
+
+
 class Build(BaseModel):
     id: int
     environment_id: int
-    specification: Specification
-    packages: List[CondaPackage]
-    status: enum.Enum
+    specification: Optional[Specification]
+    packages: Optional[List[CondaPackage]]
+    status: BuildStatus
     size: int
     scheduled_on: datetime.datetime
     started_on: Optional[datetime.datetime]
@@ -104,7 +132,7 @@ class Environment(BaseModel):
     namespace: Namespace
     name: str
     current_build_id: int
-    current_build: Build
+    current_build: Optional[Build]
 
     class Config:
         orm_mode = True
@@ -116,10 +144,20 @@ class CondaSpecificationPip(BaseModel):
 
     @validator("pip", each_item=True)
     def check_pip(cls, v):
-        try:
-            Requirement.parse(v)
-        except Exception:
-            raise ValueError(f"Invalid pypi package dependency {v}")
+
+        allowed_pip_params = ["--index-url", "--extra-index-url", "--trusted-host"]
+
+        if v.startswith("--"):
+            match = re.fullmatch("(.+?)[ =](.*)", v)
+            if match is None or match.group(1) not in allowed_pip_params:
+                raise ValueError(
+                    f"Invalid pip option '{v}' supported options are {allowed_pip_params}"
+                )
+        else:
+            try:
+                Requirement.parse(v)
+            except Exception:
+                raise ValueError(f"Invalid pypi package dependency {v}")
 
         return v
 
@@ -132,6 +170,8 @@ class CondaSpecification(BaseModel):
 
     @validator("dependencies", each_item=True)
     def check_dependencies(cls, v):
+        from conda.models.match_spec import MatchSpec
+
         if not isinstance(v, str):
             return v  # ignore pip field
 
@@ -250,3 +290,101 @@ class DockerRegistryError(enum.Enum):
         "detail": "The access controller denied access for the operation on a resource",
         "status": 403,
     }
+
+
+# API Response Objects
+class APIStatus(enum.Enum):
+    OK = "ok"
+    ERROR = "error"
+
+
+class APIResponse(BaseModel):
+    status: APIStatus
+    data: Optional[Any]
+    message: Optional[str]
+
+
+class APIPaginatedResponse(APIResponse):
+    page: int
+    size: int
+    count: int
+
+
+class APIAckResponse(BaseModel):
+    status: APIStatus
+    message: Optional[str]
+
+
+# GET /api/v1
+class APIGetStatusData(BaseModel):
+    version: str
+
+
+class APIGetStatus(APIResponse):
+    data: APIGetStatusData
+
+
+# GET /api/v1/permission
+class APIGetPermissionData(BaseModel):
+    authenticated: bool
+    entity_permissions: Dict[str, List[str]]
+    primary_namespace: str
+
+
+class APIGetPermission(APIResponse):
+    data: APIGetPermissionData
+
+
+# GET /api/v1/namespace
+class APIListNamespace(APIPaginatedResponse):
+    data: List[Namespace]
+
+
+# GET /api/v1/namespace/{name}
+class APIGetNamespace(APIResponse):
+    data: Namespace
+
+
+# GET /api/v1/environment
+class APIListEnvironment(APIPaginatedResponse):
+    data: List[Environment]
+
+
+# GET /api/v1/environment/{namespace}/{name}
+class APIGetEnvironment(APIResponse):
+    data: Environment
+
+
+# GET /api/v1/specification
+class APIGetSpecificationFormat(enum.Enum):
+    YAML = "yaml"
+    LOCKFILE = "lockfile"
+
+
+# POST /api/v1/specification
+class APIPostSpecificationData(BaseModel):
+    build_id: int
+
+
+class APIPostSpecification(APIResponse):
+    data: APIPostSpecificationData
+
+
+# GET /api/v1/build
+class APIListBuild(APIPaginatedResponse):
+    data: List[Build]
+
+
+# GET /api/v1/build/1
+class APIGetBuild(APIResponse):
+    data: Build
+
+
+# GET /api/v1/channel
+class APIListCondaChannel(APIPaginatedResponse):
+    data: List[CondaChannel]
+
+
+# GET /api/v1/package
+class APIListCondaPackage(APIPaginatedResponse):
+    data: List[CondaPackage]
